require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/finman/account/account_create'
require 'uu/finman/account/account_attributes'
require 'uu/finman/account/account_set_attributes'

# Module Unicorn Universe.
module UU
  # Module Finman.
  module Finman
    # Module of Account
    module Account
      extend self

      # Service path
      PATH = 'uu/finman/account/Account'
      # Creates a new Account. The command creates a new Account in a specified
      # location  with passed DTO parameters.<br>
      #
      # <b>Required attributes:</b><br>
      #   account_type, name, company_uri, location_uri, bank_account_number, account_currency_code (since version FINMAN3-15-8)
      # @param [UU::OS::UESURI] location_uri UESURI of the folder or organizational unit where Account will be created.
      # @param [UU::Finman::Account::AccountCreate] account DTO containing attributes of the new Account
      # @return [UU::OS::UESURI] UESURI of the new Account. In case of setting <code>dry_run</code> parameter to <code>true</code>, <code>UU::OS::UESURI::NIL_URI</code> is returned instead.
      #
      #
      # @example
      #      # Prepare folder of Organization Unit URI, where we want to create Account.
      #      location_uri = UU::OS::UESURI.new('ues:TERRITORY:FOLDER');
      #
      #      # Create Account
      #      UU::Finman::Account.create(location_uri,
      #        :name => "Name",
      #        :account_type => "BANK",
      #        :account_currency_code => "CZK",
      #        :bank_account_number => "12345",
      #        :company_uri => "ues:TERRITORY:COMPANY");
      #
      #      # Create Cash Account
      #      UU::Finman::Account.create(location_uri,
      #        :name => "Name",
      #        :account_type => "CASH",
      #        :account_currency_code => "CZK",
      #        :bank_account_number => "12345",
      #        :company_uri => "ues:TERRITORY:COMPANY");
      def create(location_uri, account)
        svc = UU::OS::REST::RemoteClient.new(Account)
        account_create = UU::Finman::Account::AccountCreate.new(account)
        transaction = transform_account_to_dto(account_create).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, transaction)
          return UU::OS::UESURI.new(res)
        end
      end

      # Obtains Account attributes from REST service and returns them
      #
      # @param [String,UU::OS::UESURI] account_uri UESURI of Account.
      #
      # @return [UU::Finman::Account::AccountAttributes] Account attributes
      #
      #
      # @example
      #      # Prepare Account URI, where we want to get attributes.
      #      account_uri = UU::OS::UESURI.new('ues:TERRITORY:ACCOUNT');
      #
      #      # Get Account attributes
      #      UU::Finman::Account.get_attributes(account_uri);
      def get_attributes(account_uri)
        svc = UU::OS::REST::RemoteClient.new(Account)
        UU::OS::QoS::QoSHandler.auto_retry do
          attrs_string = svc.get('getAttributes', account_uri)
          attrs = JSON.parse(attrs_string, :symbolize_names => true)
          attrs = remove_invalid_keys(attrs)
          return UU::Finman::Account::AccountAttributes.new(attrs)
        end
      end

      # Sets attributes of Account.
      #
      # @param [String,UU::OS::UESURI] account_uri UESURI of modified Account.
      # @param [UU::Finman::Account::AccountSetAttributes] account_set_attributes DTO containing modified attributes of the Account.
      #
      # @return [UU::OS::UESURI] UESURI of modified Account
      #
      #
      # @example
      #      # Prepare Account URI, where we want to set attributes.
      #      account_uri = UU::OS::UESURI.new('ues:TERRITORY:FOLDER');
      #
      #      # Set Account attributes.
      #      UU::Finman::Account.set_attributes(account_uri,
      #        :name => "Name");
      def set_attributes(account_uri, account_set_attributes)
        svc = UU::OS::REST::RemoteClient.new(Account)
        a_attrs = UU::Finman::Account::AccountSetAttributes.new(account_set_attributes)
        transaction = transform_account_to_dto(a_attrs).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', account_uri, transaction)
          return UU::OS::UESURI.new(res)
        end
      end

      # Sets state of Account.
      #
      # @param [String,UU::OS::UESURI] account_uri UESURI of Account
      # @param [UU::Finman::Account::AccountSetState] state DTO containing details of the state change
      #
      #
      # @example
      #      # Prepare Account URI, where we want to set state.
      #      account_uri = UU::OS::UESURI.new('ues:TERRITORY:ACCOUNT');
      #
      #      # Set state of Account.
      #      UU::Finman::Account.set_state(account_uri, :account_state => "ACTIVE" ,:comment=>"Some sample comment");
      def set_state(account_uri, state)
        svc = UU::OS::REST::RemoteClient.new(Account)
        payload = UU::Finman::Account::AccountSetState.new(state).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setState', account_uri, payload)
        end
      end

      # Adjust dto of account to match requirements of account dto of server side.
      # @private
      def transform_account_to_dto(acc_dto)
        if acc_dto.is_a?(UU::OS::REST::DTO)
          attributes = acc_dto.to_hash
          if attributes[:accountType] == "CASH"
            attributes.delete(:authorizedOverdraft)
            attributes.delete(:registeredAccount)
            attributes.delete(:daysExpectedPaydateBo)
            attributes.delete(:maxPaymentsBankOrdersArtifact)
            attributes.delete(:bankOrdersOwnerUri)
            attributes.delete(:bankOrdersFolderUri)
            attributes.delete(:numberOfReferencedBO)
            attributes.delete(:iban)
          end
          return attributes
        end
        fail "Given object is neither UU::OS::REST::DTO nor its descendant!"
      end
      private :transform_account_to_dto


      # Hash returned from service contains all attributes from cash account.
      # Remove invalid keys to contain only values for this specific class of account.
      # @private
      def remove_invalid_keys(hash)
        if hash.is_a?(Hash)
          if hash[:accountType] == "CASH"
            hash.delete(:authorizedOverdraft)
            hash.delete(:registeredAccount)
            hash.delete(:daysExpectedPaydateBo)
            hash.delete(:maxPaymentsBankOrdersArtifact)
            hash.delete(:bankOrdersOwnerUri)
            hash.delete(:bankOrdersFolderUri)
            hash.delete(:numberOfReferencedBO)
            hash.delete(:iban)
          end
          return hash
        end
        fail "Given object must be Hash!"
      end
      private :remove_invalid_keys

    end
  end
end
