require 'uu/os/rest/dto'
require 'uu/os/uesuri'

module UU
  module Finman
    module Account

      # DTO for create command of account.<br>
      class AccountCreate < UU::OS::REST::DTO

        # Type of Account.
        # Required attribute. Available values are BANK or CASH
        # @param [UU::Finman::Account::AccountType]
        # @return [UU::Finman::Account::AccountType]
        attr_accessor :account_type

        # Name of created account. Its value is automatically trimmed.
        # Cannot be null nor empty string nor longer than 250 characters and can contain only characters allowed for BTName.
        # The create command fails otherwise.
        # @param [String]
        # @return [String]
        attr_accessor :name

        # Code of created account. Its value is automatically trimmed.
        # The code must be unique in target territory. It cannot be empty string nor longer than 250 characters
        # and can contain only characters allowed for BTCode. The create command will fail otherwise.
        # When null is set, the code is generated automatically.
        # @param [String]
        # @return [String]
        attr_accessor :code

        # Description of created account.
        # When null is set, the description is taken from the meta artifact. When the meta artifact does not define it, it remains empty.
        # @param [String]
        # @return [String]
        attr_accessor :description

        # URI of the role competent for created account.
        # When null is set, competent role will be taken from authorization.
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :competent_role_uri

        # Security level of created account.
        # When null is set, the Security level is taken from the meta artifact. When the meta artifact does not define it, it remains empty.
        # @param [String]
        # @return [String]
        attr_accessor :security_level

        # URI of company of created Account.
        # When null is set, the create command will fail.
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :company_uri

        # Currency of created account.
        # Required attribute since version FINMAN3-15-8.
        # @param [String]
        # @return [String]
        attr_accessor :account_currency_code

        # Bank account number of created account.
        # When null is set, the create command will fail.
        # @param [String]
        # @return [String]
        attr_accessor :bank_account_number

        # Iban number of created account.
        # @param [String]
        # @return [String]
        attr_accessor :iban

        # Authorized overdraft of created account.
        # @param [Fixnum]
        # @return [Fixnum]
        attr_accessor :authorized_overdraft

        # Minimal balance in created account.
        # @param [Fixnum]
        # @return [Fixnum]
        attr_accessor :minimal_balance

        # Note of created account.
        # @param [String]
        # @return [String]
        attr_accessor :note

        # Registered account of created account.
        # @param [TrueClass, FalseClass]
        # @return [TrueClass, FalseClass]
        # This parameter cannot be used for Cash Account.
        attr_accessor :registered_account

        # Bank order owner uri of created account
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :bank_orders_owner_uri

        # Bank orders folder uri of created account
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :bank_orders_folder_uri

        # Days expected paydate bank order of created account
        # @param [Fixnum]
        # @return [Fixnum]
        # This parameter cannot be used for Cash Account.
        attr_accessor :days_expected_paydate_bo

        # Maximal payments bank orders Artifact of created account
        # @param [Fixnum]
        # @return [Fixnum]
        attr_accessor :max_payments_bank_orders_artifact

        # Number of referenced bank orders.
        # @param [Fixnum]
        # @return [Fixnum]
        attr_accessor :number_of_referenced_bo

        # Dry run. If true, all validations will be processed, but account
        # will not be created.
        # @param [TrueClass, FalseClass]
        # @return [TrueClass, FalseClass]
        attr_accessor :dry_run

        # Debit Teller's Manager
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :debit_tellers_manager_uri

        # Debit Teller
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :debit_teller_uri

        # Outgoing Payment Checker's Manager
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :outgoing_payment_checkers_manager_uri

        # Outgoing Payment Checker
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :outgoing_payment_checker_uri

        # Incoming Payment Checker's Manager
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :incoming_payment_checkers_manager_uri

        # Incoming Payment Checker
        # @param [String, UU::OS::UESURI]
        # @return [UU::OS::UESURI]
        attr_accessor :incoming_payment_checker_uri


        def account_type
          attributes[:accountType]
        end

        def account_type=(val)
          attributes[:accountType] = val
        end

        def name
          attributes[:name]
        end

        def name=(val)
          attributes[:name] = val
        end

        def code
          attributes[:code]
        end

        def code=(val)
          attributes[:code] = val
        end

        def description
          attributes[:description]
        end

        def description=(val)
          attributes[:description] = val
        end

        def competent_role_uri
          attributes[:competentRoleUri]
        end

        def competent_role_uri=(val)
          if val
            attributes[:competentRoleUri] = UU::OS::UESURI.new(val)
          else
            attributes[:competentRoleUri] = nil
          end
        end

        def security_level
          attributes[:securityLevel]
        end

        def security_level=(val)
          attributes[:securityLevel] = val
        end

        def company_uri
          attributes[:companyUri]
        end

        def company_uri=(val)
          if val
            attributes[:companyUri] = UU::OS::UESURI.new(val)
          else
            attributes[:companyUri] = nil
          end
        end


        def account_currency_code
          attributes[:accountCurrencyCode]
        end

        def account_currency_code=(val)
          attributes[:accountCurrencyCode] = val
        end


        def bank_account_number
          attributes[:bankAccountNumber]
        end

        def bank_account_number=(val)
          attributes[:bankAccountNumber] = val
        end


        def iban
          attributes[:iban]
        end

        def iban=(val)
          attributes[:iban] = val
        end


        def authorized_overdraft
          attributes[:authorizedOverdraft]
        end

        def authorized_overdraft=(val)
          attributes[:authorizedOverdraft] = val
        end


        def minimal_balance
          attributes[:minimalBalance]
        end

        def minimal_balance=(val)
          attributes[:minimalBalance] = val
        end


        def note
          attributes[:note]
        end

        def note=(val)
          attributes[:note] = val
        end


        def registered_account
          attributes[:registeredAccount]
        end

        def registered_account=(val)
          attributes[:registeredAccount] = val
        end


        def bank_orders_owner_uri
          attributes[:bankOrdersOwnerUri]
        end

        def bank_orders_owner_uri=(val)
          if val
            attributes[:bankOrdersOwnerUri] = UU::OS::UESURI.new(val)
          else
            attributes[:bankOrdersOwnerUri] = nil
          end
        end


        def bank_orders_folder_uri
          attributes[:bankOrdersFolderUri]
        end

        def bank_orders_folder_uri=(val)
          if val
            attributes[:bankOrdersFolderUri] = UU::OS::UESURI.new(val)
          else
            attributes[:bankOrdersFolderUri] = nil
          end
        end


        def days_expected_paydate_bo
          attributes[:daysExpectedPaydateBO]
        end

        def days_expected_paydate_bo=(val)
          attributes[:daysExpectedPaydateBO] = val
        end


        def max_payments_bank_orders_artifact
          attributes[:maxPaymentsBankOrdersArtifact]
        end

        def max_payments_bank_orders_artifact=(val)
          attributes[:maxPaymentsBankOrdersArtifact] = val
        end

        def number_of_referenced_bo
          attributes[:numberOfReferencedBO]
        end

        def number_of_referenced_bo=(val)
          attributes[:numberOfReferencedBO] = val
        end

        def dry_run
          attributes[:dryRun]
        end

        def dry_run=(val)
          attributes[:dryRun] = val
        end

            def debit_tellers_manager_uri
              attributes[:debitTellersManagerUri]
            end


            def debit_tellers_manager_uri=(val)
              if val
                attributes[:debitTellersManagerUri] = UU::OS::UESURI.new(val)
              else
                attributes[:debitTellersManagerUri] = nil
              end
            end



            def debit_teller_uri
              attributes[:debitTellerUri]
            end

            def debit_teller_uri=(val)
              if val
                attributes[:debitTellerUri] = UU::OS::UESURI.new(val)
              else
                attributes[:debitTellerUri] = nil
              end
            end


            def outgoing_payment_checkers_manager_uri
              attributes[:outgoingPaymentCheckersManagerUri]
            end

            def outgoing_payment_checkers_manager_uri=(val)
             if val
               attributes[:outgoingPaymentCheckersManagerUri] = UU::OS::UESURI.new(val)
             else
               attributes[:outgoingPaymentCheckersManagerUri] = nil
             end
           end


            def outgoing_payment_checker_uri
              attributes[:outgoingPaymentCheckerUri]
            end

            def outgoing_payment_checker_uri=(val)
		if val
			attributes[:outgoingPaymentCheckerUri] = UU::OS::UESURI.new(val)
		else
			attributes[:outgoingPaymentCheckerUri] = nil
		end
		end


            def incoming_payment_checkers_manager_uri
              attributes[:incomingPaymentCheckersManagerUri]
            end

            def incoming_payment_checkers_manager_uri=(val)
             if val
               attributes[:incomingPaymentCheckersManagerUri] = UU::OS::UESURI.new(val)
             else
               attributes[:incomingPaymentCheckersManagerUri] = nil
             end
           end


            def incoming_payment_checker_uri
              attributes[:incomingPaymentCheckerUri]
            end

        def incoming_payment_checker_uri=(val)
		    if val
		    attributes[:incomingPaymentCheckerUri] = UU::OS::UESURI.new(val)
		    else
		    attributes[:incomingPaymentCheckerUri] = nil
		    end
		end

      end
    end
  end
end

