require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/artifact/artifact_state_type'
require 'uu/finman/financialtransaction/financial_transaction_get_transaction_list'
require 'uu/finman/financialtransaction/financial_transaction_attributes'
require 'uu/finman/financialtransaction/financial_transaction_pay_transactions_to_be_paid'
require 'uu/finman/financialtransaction/financial_transaction_check_transactions_to_be_paid'
require 'uu/finman/financialtransaction/financial_transaction_set_state'

# Module Unicorn Universe
module UU
  # Module Finman
  module Finman
    # Module of finman Financial transaction
    module FinancialTransaction
      extend self

      # Service path
      PATH = 'uu/finman/financialtransaction/FinancialTransaction'
      # Retrieves a result list of financial transactions for specified business case by UESURI.
      # The resulting attachment list is filtered according to the criteria provided by the criteria attribute.<br>
      #
      # Allowed values for attribute <code>businessCaseSide</code> in query are <code>CREDIT</code> for filtering by credit business case side and <code>DEBIT</code> for filtering by debit business case side. Use it for filtering transaction only by CREDIT / DEBIT side of Business Case(same as main entity). If parameter not used, returns transaction with CREDIT or DEBIT side.<br>
      # For allowed values for attribute <code>stateType</code> in query, look at UU::OS::Artifact::ArtifactStateType
      #
      # @param [String, UU::OS::UESURI] business_case_uri UESURI of the business case.
      # @param [UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionList] criteria DTO Financial_transaction_get_transaction_list containing paging and filtering criteria. Available parameters are:<br> {FinancialTransactionAttributes#creation_time creationTime}, {FinancialTransactionAttributes#credit_vat_date creditVatDate}, {FinancialTransactionAttributes#debit_vat_date debitVatDate}, {FinancialTransactionAttributes#meta_artifact_uri metaArtifactUri}, {FinancialTransactionAttributes#pay_date payDate}, {FinancialTransactionAttributes#due_date dueDate}, {FinancialTransactionAttributes#state_name stateName}, {FinancialTransactionAttributes#state_type stateType}, {FinancialTransactionAttributes#cost_item_code costItemCode}, {FinancialTransactionAttributes#income_item_code incomeItemCode}, {FinancialTransactionAttributes#transaction_currency transactionCurrency}, businessCaseSide.
      # @option criteria [String] :query The UES query string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there.
      #
      # @return [UU::OS::REST::ResultList<UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionList, UU::Finman::FinancialTransaction::FinancialTransactionAttributes>] List of financial transaction.
      #
      #
      # @example
      #      # Prepare Business Case URI, where we want to get Financial Transaction list.
      #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:BUSINESS_CASE');
      #
      #      # Get transaction list on  specific Business Case
      #      UU::Finman::FinancialTransaction.get_transaction_list(business_case_uri,
      #         :query => "creationTime > '2015-01-01' AND metaArtifactUri = 'ues:TERRITORY:META_ARTIFACT_OF_FINANCIAL_TRANSACTION' AND stateType = ACTIVE");
      def get_transaction_list(bc_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        dto = UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getTransactionList', bc_uri)
          return UU::OS::REST::ResultList.new(UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionList, UU::Finman::FinancialTransaction::FinancialTransactionAttributes, res)
        end
      end

      # Deletes the specified financial transaction artifact from the system.
      # The financial transaction artifact cannot be deleted when the user executing this command is not cast in the Service Executives group.
      #
      # @param [String, UU::OS::UESURI] fintr_uri UESURI of the financial transaction which is about to be deleted.
      #
      # @example
      #   # Delete financial transaction with specified URI.
      #   UU::Finman::FinancialTransaction.delete('ues:TERRITORY:P1234567890')
      def delete(fintr_uri)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('delete', fintr_uri)
        end
      end

      # Closes the specified financial transaction artifact.
      # The financial transaction artifact cannot be closed when the user executing this command is not cast in Group Role cast in Accounting Authorities.
      #
      # @param [String, UU::OS::UESURI] fintr_uri UESURI of the financial transaction which is about to be closed.
      #
      # @example
      #   # Close financial transaction with specified URI.
      #   UU::Finman::FinancialTransaction.close('ues:TERRITORY:P1234567890')
      def close(fintr_uri)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('close', fintr_uri)
        end
      end

      # Returns list of Financial Transactions related to the account by debit side that are matching the query criteria, specified by account UESURI and must have Pay Payment acivity.
      # The resulting attachment list is filtered according to the criteria provided by the criteria attribute.<br>
      #
      # Allowed values for attribute <code>transactionType</code> in query are <code>CASH</code> for show only NONCASH payments, <code>NONCASH</code> for show only NONCASH payments and <code>ALL</code> for both types. If parameter not used, returns transaction with type CASH or NONCASH .<br>
      #
      # @param [String, UU::OS::UESURI] account_uri UESURI of the account.
      # @param [UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionToBePaidList] criteria DTO Financial_transaction_get_transaction_to_be_paid_list containing paging and filtering criteria. Available parameters are:<br> {FinancialTransactionToBePaidAttributes#due_date dueDate, transactionType}.
      # @option criteria [String] :query The UES query string. It has to conform the ues query syntax (see documentation for more info). Filtering and order of the result list can be specified there.
      #
      # @return [UU::OS::REST::ResultList<UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionToBePaidList, UU::Finman::FinancialTransaction::FinancialTransactionToBePaidAttributes>] List of financial transactions.
      #
      #
      # @example
      #      # Prepare account URI, where we want to get Financial Transaction list.
      #      account_uri = UU::OS::UESURI.new('ues:TERRITORY:ACCOUNT_URI');
      #
      #      # Get transaction list on specific Account
      #      UU::Finman::FinancialTransaction.get_transaction_to_be_paid_list(account_uri,
      #         :query => "dueDate < '2016-01-01' AND dueDate > '2014-01-01'");
      #
      #      # Get transaction list on specific Account
      #      UU::Finman::FinancialTransaction.get_transaction_to_be_paid_list(account_uri,
      #         :query => "dueDate < '2016-01-01' AND dueDate > '2014-01-01' AND transactionType = 'CASH'");
      def get_transaction_to_be_paid_list(acc_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        dto = UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionToBePaidList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getTransactionToBePaidList', acc_uri)
          return UU::OS::REST::ResultList.new(UU::Finman::FinancialTransaction::FinancialTransactionGetTransactionToBePaidList, UU::Finman::FinancialTransaction::FinancialTransactionToBePaidAttributes, res)
        end
      end
      
      
      # Processes financial transactions to be paid. Sets the Pay activity to final state and sets the financial transaction to Money On The Way State.
      # The paying process is asynchronous. It is possible to wait for the end of the process. More information in [UU::OS::Env::Process] and in [UU::OS::REST::Future]. The results file will be stored in attachment. Attachment UESURI will be returned as the process result.
      #
      # Input file format: CSV with one column and no header. The column contains UESURIs of all financial transactions to be paid.
      #
      # Output file format:CSV file separated by semicolon with header. See UNI-BT:UU.FINMAN/SWA/CMD_API/MODULE_FINTX/ENTITY_FINTX for more details.
      #
      # @param [String, UU::OS::UESURI] account_uri UESURI of the account.
      # @param [UU::Finman::FinancialTransaction::FinancialTransactionPayTransactionsToBePaid] DTO Financial_transaction_pay_transactions_to_be_paid containing the data file.
      #
      # @return [UU::OS::UESURI] UESURI of the asynchronous process which can be used to wait for the results file.  
      #
      # @example
      #      # Prepare account URI, where we want to get Financial Transaction list.
      #      account_uri = UU::OS::UESURI.new('ues:TERRITORY:ACCOUNT_URI');
      #
      #      # Start the paying
      #      File.open('payments.csv', 'r') do |f|
      #        # Call the command
      #        process =  UU::Finman::FinancialTransaction.pay_transactions_to_be_paid(account_uri, :financial_transactions => f)
      #
      #        # Initialize the future
      #        future = UU::OS::REST::Future.new(process)
      #
      #        # Get the UESURI of the attachment from process result. 
      #        result = future.get(nil, 60)
      #      end
      #
      def pay_transactions_to_be_paid(acc_uri, params)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        #svc = UU::OS::REST::RemoteClient.new(UU::OS::Sheet)
        dto = UU::Finman::FinancialTransaction::FinancialTransactionPayTransactionsToBePaid.new(params)
        payload = dto.to_hash
        payload.delete(:financial_transactions)
        tmpfile = Tempfile.new("PayTransactionsToBePaid")
        tmpfile.binmode
        begin
          if (!dto.financial_transactions.kind_of?(UU::OS::REST::BinaryValue))
            dto.financial_transactions = UU::OS::REST::BinaryValue.new(dto.financial_transactions)
          end
          if (dto.financial_transactions.data.respond_to?(:read))
            if (dto.financial_transactions.data.respond_to?(:path))
              payload[:financialTransactions] = dto.financial_transactions.to_hash(false)
            else
              dto.financial_transactions.data.rewind
              tmpfile.write(dto.financial_transactions.data.read)
              payload[:financialTransactions] = tmpfile
            end
          else
            tmpfile.write(dto.financial_transactions.data)
            payload[:financialTransactions] = tmpfile
          end
          tmpfile.flush
          tmpfile.rewind

          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('payTransactionsToBePaid', acc_uri, payload)
            return UU::OS::UESURI.new(res)            
          end
        ensure
        tmpfile.close
        tmpfile.unlink
        end
      
      end
      
      
      
      # Processes financial transactions to be paid. Sets the Check activity to requested state with optional annotation.
      # The checking process is asynchronous. It is possible to wait for the end of the process. More information in [UU::OS::Env::Process] and in [UU::OS::REST::Future]. The results file will be stored in attachment. Attachment UESURI will be returned as the process result.
      #
      # Input file format: CSV with 3 columns: TRANSACTION_URI, STATE, ANNOTATION separated by semicolon. 
      # The column TRANSACTION_URI contains UESURIs of all financial transactions to be checked. 
      # The column STATE contains requested state of the activity, which can be YES_ACTIVE or NO_ACTIVE (any other value results in error INCORRECT_ACTIVITY_STATE).
      # The column ANNOTATION contains requested annotation. It can be empty, in this case no annotation will be set.
      #
      # Output file format:CSV file separated by semicolon with header. See UNI-BT:UU.FINMAN/SWA/CMD_API/MODULE_FINTX/ENTITY_FINTX for more details.
      #
      # @param [String, UU::OS::UESURI] account_uri UESURI of the account.
      # @param [UU::Finman::FinancialTransaction::FinancialTransactionCheckTransactionsToBePaid] DTO Financial_transaction_check_transactions_to_be_paid containing the data file.
      #
      # @return [UU::OS::UESURI] UESURI of the asynchronous process which can be used to wait for the results file.  
      #
      # @example
      #      # Prepare account URI, where we want to get Financial Transaction list.
      #      account_uri = UU::OS::UESURI.new('ues:TERRITORY:ACCOUNT_URI');
      #
      #      # Start the paying
      #      File.open('payments.csv', 'r') do |f|
      #        # Call the command
      #        process =  UU::Finman::FinancialTransaction.check_transactions_to_be_paid(account_uri, :financial_transactions => f)
      #
      #        # Initialize the future
      #        future = UU::OS::REST::Future.new(process)
      #
      #        # Get the UESURI of the attachment from process result. 
      #        result = future.get(nil, 60)
      #      end
      #
      def check_transactions_to_be_paid(acc_uri, params)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        #svc = UU::OS::REST::RemoteClient.new(UU::OS::Sheet)
        dto = UU::Finman::FinancialTransaction::FinancialTransactionCheckTransactionsToBePaid.new(params)
        payload = dto.to_hash
        payload.delete(:financial_transactions)
        tmpfile = Tempfile.new("CheckTransactionsToBePaid")
        tmpfile.binmode
        begin
          if (!dto.financial_transactions.kind_of?(UU::OS::REST::BinaryValue))
            dto.financial_transactions = UU::OS::REST::BinaryValue.new(dto.financial_transactions)
          end
          if (dto.financial_transactions.data.respond_to?(:read))
            if (dto.financial_transactions.data.respond_to?(:path))
              payload[:financialTransactions] = dto.financial_transactions.to_hash(false)
            else
              dto.financial_transactions.data.rewind
              tmpfile.write(dto.financial_transactions.data.read)
              payload[:financialTransactions] = tmpfile
            end
          else
            tmpfile.write(dto.financial_transactions.data)
            payload[:financialTransactions] = tmpfile
          end
          tmpfile.flush
          tmpfile.rewind

          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('checkTransactionsToBePaid', acc_uri, payload)
            return UU::OS::UESURI.new(res)            
          end
        ensure
        tmpfile.close
        tmpfile.unlink
        end
      
      end      

      
      # Forces update of financial transaction attributes on satellite module.
      # Can be executed only for original financial transaction. Propagates attributes to PSM and SSM and also to copy if main entity is ITA transaction.
      #
      # @param [String, UU::OS::UESURI] fintr_uri UESURI of the financial transaction which should be updated on SM.
      #
      # @example
      #   # Force update of financial transaction with specific URI.
      #   UU::Finman::FinancialTransaction.force_update_attributes('ues:TERRITORY:P1234567890')
      def force_update_attributes(fintr_uri)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('forceUpdateAttributes', fintr_uri)
          return UU::OS::UESURI.new(res)    
        end
      end 
      
      # Forces update of financial transaction assignments on satellite module.
      # Can be executed for any financial transaction, either original or copy. Triggers recomputing of assignments to BC and Date dimensions. Update is done even for closed transactions. 
      # Does not propagate to opposite territory for ITA transactions.
      # Main purpose is to recompute assignments of closed transactions to BC dimensions when organizational structure changes (because assignments of closed payments are normally not updated).  
      #
      # @param [String, UU::OS::UESURI] fintr_uri UESURI of the financial transaction which should reassigned on SSM.
      #
      # @example
      #   # Force update of financial transaction assignments with specific URI.
      #   UU::Finman::FinancialTransaction.force_update_assignments('ues:TERRITORY:P1234567890')
      def force_update_assignments(fintr_uri)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('forceUpdateAssignments', fintr_uri)
          return UU::OS::UESURI.new(res)    
        end
      end

      # Advance the specified financial transaction to a specific state
      # The financial transaction state cannot be set when the user executing this command is not cast in the Authorised Executives group.
      #
      # @param [String, UU::OS::UESURI] fintr_uri UESURI of the financial transaction which want to advanced to the state
      # @param [String, financialTransactionSetState] financialTransactionSetState of the financial transaction state which want to be advanced
      #
      # @example
      #   # Set advanced state of financial transaction with specified URI.
      #   UU::Finman::FinancialTransaction.setState('ues:TERRITORY:P1234567890',
      #        :financial_transaction_state => "INVOICE_RECEIVED");
      def set_state(fintr_uri, financialTransactionSetState)
        svc = UU::OS::REST::RemoteClient.new(FinancialTransaction)
        transaction = UU::Finman::FinancialTransaction::FinancialTransactionSetState.new(financialTransactionSetState).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setState', fintr_uri, transaction)
        end
      end

    end
  end
end
